"use strict";
/**
 * Copyright (c) 2018-present, tarant
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fiber_1 = tslib_1.__importDefault(require("../fiber/fiber"));
const actor_proxy_1 = tslib_1.__importDefault(require("./actor-proxy"));
const actor_system_configuration_builder_1 = tslib_1.__importDefault(require("./configuration/actor-system-configuration-builder"));
const function_1 = tslib_1.__importDefault(require("./fn/function"));
class ActorSystem {
    static for(configuration) {
        return new ActorSystem(configuration);
    }
    static default() {
        return new ActorSystem(actor_system_configuration_builder_1.default.define().done());
    }
    constructor(configuration) {
        this.requirements = ['default'];
        this.actors = new Map();
        this.subscriptions = new Map();
        const { mailbox, resources, tickInterval, materializers, resolvers, supervisor } = configuration;
        this.mailbox = mailbox;
        this.materializers = materializers;
        this.resolvers = resolvers;
        this.supervisor = supervisor;
        this.fiber = fiber_1.default.with({ resources, tickInterval });
        this.fiber.acquire(this);
    }
    process() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.actors.forEach((v) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield this.mailbox.poll(this.subscriptions.get(v.id));
            }));
        });
    }
    /**
     * Stops the actor system and frees all actors
     */
    free() {
        setTimeout(() => this.fiber.free(), 0);
    }
    /**
     * Creates a new root actor.
     *
     * @param classFn Constructor of the actor to create
     * @param values Parameters to pass to the constructor
     */
    actorOf(ClassFn, values) {
        const instance = new ClassFn(...values);
        const proxy = actor_proxy_1.default.of(this.mailbox, instance);
        const subscription = this.mailbox.addSubscriber(instance);
        this.actors.set(instance.id, instance);
        this.subscriptions.set(instance.id, subscription);
        this.setupInstance(instance, proxy);
        return proxy;
    }
    /**
     * Looks for an existing actor in this actor system. If none exist,
     * it will try to resolve the actor, by id, with the provided resolvers.
     *
     * @param id Id of the actor to find
     */
    actorFor(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let instance = this.actors.get(id);
            if (instance === undefined) {
                for (const resolver of this.resolvers) {
                    try {
                        instance = (yield resolver.resolveActorById(id));
                        break;
                    }
                    catch (_) {
                        //
                    }
                }
                if (instance === undefined) {
                    return Promise.reject(new Error(`unable to resolve actor ${id}`));
                }
                this.actors.set(id, instance);
                this.subscriptions.set(instance.id, this.mailbox.addSubscriber(instance));
                const proxy = actor_proxy_1.default.of(this.mailbox, instance);
                this.setupInstance(instance, proxy);
                return proxy;
            }
            return actor_proxy_1.default.of(this.mailbox, instance);
        });
    }
    /**
     * Converts a function to a function actor, with the same signature. Function actors behave the same way as
     * actors, but can be called as ordinary functions.
     *
     * Like actors, there is a limitation on the signature of the function. It can receive any number and type
     * of parameters, but it *must* return a Promise.
     *
     * @param fn Function to be wrapped in an actor
     */
    functionFor(fn) {
        return function_1.default.for(this, fn);
    }
    /**
     * Tries to find an actor, if it doesn't exist, creates a new one.
     *
     * @param id Id of the actor to find
     * @param elseClass Constructor of the actor to create, if doesn't exist
     * @param values Parameters of the constructor
     */
    resolveOrNew(id, elseClass, values) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.actorFor(id);
            }
            catch (_) {
                return this.actorOf(elseClass, values);
            }
        });
    }
    setupInstance(instance, proxy) {
        instance.self = proxy;
        instance.system = this;
        instance.materializers = this.materializers;
        instance.supervisor = this.supervisor;
        instance.initialized();
    }
}
exports.default = ActorSystem;
