"use strict";
/**
 * Copyright (c) 2018-present, tarant
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const uuid_1 = tslib_1.__importDefault(require("../helper/uuid"));
const actor_proxy_1 = tslib_1.__importDefault(require("./actor-proxy"));
/**
 * Class that must be extended by all actors. All defined public methods in actors should be
 * asynchronous (return a Promise<T>) or return void.
 */
class Actor {
    constructor(id) {
        this.self = this;
        this.materializers = [];
        this.scheduled = new Map();
        this.topicSubscriptions = new Map();
        this.busy = false;
        this.id = id || (0, uuid_1.default)();
        this.partitions = [this.id];
    }
    /**
     * Method called by the mailbox when there are messages to be processed. This should
     * not be overriden by the actor.
     *
     * @param message Message received from the mailbox
     */
    onReceiveMessage(message) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.busy) {
                return false;
            }
            this.busy = true;
            const actorMessage = message.content;
            try {
                this.materializers.forEach((materializer) => materializer.onBeforeMessage(this, actorMessage));
                const result = yield this.dispatchAndPromisify(actorMessage);
                actorMessage.resolve(result);
            }
            catch (ex) {
                this.materializers.forEach((materializer) => materializer.onError(this, actorMessage, ex));
                const strategy = yield this.supervisor.supervise(this.self, ex, actorMessage);
                if (strategy === 'drop-message') {
                    actorMessage.reject(ex);
                    return true;
                }
                else if (strategy === 'retry-message') {
                    return false;
                }
                else {
                    actorMessage.reject(ex);
                    return true;
                }
            }
            finally {
                this.busy = false;
                this.materializers.forEach((materializer) => materializer.onAfterMessage(this, actorMessage));
            }
            return true;
        });
    }
    /**
     * Supervision method, called when a child actor failed.
     * The default strategy is to delegate the supervision to this actor supervisor.
     *
     * @param actor Actor that raised the exception
     * @param exception Exception that raised the actor
     * @param message Message that we failed to process
     */
    supervise(actor, exception, message) {
        return this.supervisor.supervise(actor, exception, message);
    }
    /**
     * Schedules a message to this actor every {interval} milliseconds. Returns a cancellable, that
     * can be passed to #cancel to stop the scheduled message.
     *
     * @param interval Interval, in ms, between messages
     * @param fn Message to send to this current actor, in form of a method reference (like this.myMethod)
     * @param values Parameters to pass with the message (parameters of the method to call)
     *
     * @see Actor#cancel
     */
    schedule(interval, fn, values) {
        const id = (0, uuid_1.default)();
        setTimeout(() => {
            const sysAny = this.system;
            this.scheduled.set(id, setInterval(() => actor_proxy_1.default.sendAndReturn(sysAny.mailbox, this.id, fn.name, values), interval));
        }, 0);
        return id;
    }
    /**
     * Schedules a message to this actor once, after {timeout} milliseconds. Returns a cancellable, that
     * can be passed to #cancel to stop the scheduled message.
     *
     * @param timeout Time to send the message, in ms
     * @param fn Message to send to this current actor, in form of a method reference (like this.myMethod)
     * @param values Parameters to pass with the message (parameters of the method to call)
     *
     * @see Actor#cancel
     */
    scheduleOnce(timeout, fn, values) {
        const id = (0, uuid_1.default)();
        setTimeout(() => {
            const sysAny = this.system;
            this.scheduled.set(id, setTimeout(() => {
                actor_proxy_1.default.sendAndReturn(sysAny.mailbox, this.id, fn.name, values);
                this.scheduled.delete(id);
            }, timeout));
        }, 0);
        return id;
    }
    /**
     * Cancels a scheduled action created by #schedule or #scheduleOnce
     *
     * @param cancellable Cancellable reference
     * @see Actor#schedule
     * @see Actor#scheduleOnce
     */
    cancel(cancellable) {
        setTimeout(() => {
            const id = this.scheduled.get(cancellable);
            clearTimeout(id);
            clearInterval(id);
            this.scheduled.delete(cancellable);
        }, 0);
    }
    /**
     * Creates a child actor of this actor. The current actor will behave as the supervisor
     * of the created actor.
     *
     * @param classFn Constructor of the actor to build
     * @param values Values to pass as the constructor parameters
     */
    actorOf(classFn, values) {
        const actor = this.system.actorOf(classFn, values);
        actor.ref.supervisor = this;
        return actor;
    }
    subscribeToTopic(topic) {
        setTimeout(() => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const topicSubscription = yield topic.subscribe(this);
            this.topicSubscriptions.set(topic.id, topicSubscription);
        }), 0);
    }
    unsubscribeFromTopic(topic) {
        const id = this.topicSubscriptions.get(topic.id);
        if (id === undefined) {
            return;
        }
        topic.unsubscribe(id);
        this.topicSubscriptions.delete(topic.id);
    }
    dispatchAndPromisify(actorMessage) {
        try {
            const r = this.constructor.prototype[actorMessage.methodName].apply(this, actorMessage.arguments);
            if (r && r.then && r.catch) {
                return r;
            }
            else {
                return Promise.resolve(r);
            }
        }
        catch (ex) {
            return Promise.reject(ex);
        }
    }
    initialized() {
        this.materializers.forEach((materializer) => materializer.onInitialize(this));
    }
}
exports.default = Actor;
